function [SENSE, SPEC, OPTHRESH, AUC, PLR, TSS] = ROC_GRAPH(neg, pos, COLOR)
if nargin < 3
    COLOR = 'k';
end
On1Sens = [];
On1Spec = [];
On1realSpec = [];
NLR = 1;
PLR = 0;
minthresh = min([min(pos),min(neg)]);
maxthresh = max([max(pos),max(neg)]);
stepsize = 0.001;
TSS = [];
for thresh = minthresh:stepsize:maxthresh
    TrueHits        = length(find(neg<thresh)); %correct reject low ps
    Sensitivity     = TrueHits/length(neg);
    TrueMiss        = length(find(pos>thresh)); %correctly keep high ps
    Specificity     = TrueMiss/length(pos);
    AntiSpecificity = (1-Specificity);
    On1Sens         = [On1Sens Sensitivity];
    On1Spec         = [On1Spec AntiSpecificity]; %1-specificity
    On1realSpec     = [On1realSpec Specificity]; %save specificity values
    
    %maximum positive likelihood ratio
    %enter this loop if the specifity does not equal 1 (which would make 
    %the PLR = 0), and if the current PLR is greater than the previous one.
    %this loop will save over itself with the maximum PLR and the 
    %associated negative PLR value
    if Specificity ~= 1 && Sensitivity/(1-Specificity) > PLR 
        PLR = Sensitivity/(1-Specificity);     %positive likelihood ratio
        NLR = (1 - Sensitivity)/(Specificity); %negative likelihood ratio
        SENSE = Sensitivity;                   %sensitivity
        SPEC = Specificity;                    %specificity
        OPTHRESH = thresh;                     %threshold 
    end
    
    Youden = Sensitivity + Specificity - 1;
    TSS = [TSS ; thresh, Sensitivity, Specificity, Youden];
end

AUC = 0;
for inter = 2:length(On1Spec)
    AUC = AUC + (On1Spec(inter)-On1Spec(inter-1)) * ...
                (.5*(On1Sens(inter-1) + On1Sens(inter)));
end

% figure
plot(On1Spec, On1Sens, 'Color', COLOR, 'LineWidth', 2)
ylabel('Sensitivity')
xlabel('1 - Specificity')
hold on
% plot([0 1], [0 1], 'Color', [.5 .5 .5], 'LineStyle', ':')
axis([-.01 1.01 -.01 1.01])

%AUC (area under the curve) was determined via numerical integration 
%(trapezoidal rule) for each detection scheme. Further, positive 
%likelihood ratios were calculated as the sensitivity / (1  specificity) 
%Positive likelihood ratios of 4-5 indicate moderate to high diagnostic 
%value
end