function [] = find_RFFcycles_ACC_final(audiofolname,filename)
%%%%%%%%%V.1 for ACC%%%%%%%%%%%%%
clear aRFF; close all;
global perform_fol; 

%declaring subject folders
aRFF.subjectid = filename(1:end-4);
folname = [perform_fol aRFF.subjectid '\'];
varfolname = [folname 'var\'];
fricfile = [varfolname 'fric_param.mat'];

%if we do not have the fricfile, need to go back and create it
if ~exist(fricfile,'file')
    find_fric_halcyon_ACC(audiofolname, filename)
    return;
else
    load(fricfile);
end


diff_prob = smooth(f_prob - v_prob,round(aRFF.fs/20));



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Initialize Threshvect. This is currently tuned to manual RFF estimates.
offshiftthresh = -.10; %-0.10 change thresh by 10%, + gets closer to VC, - gets further from VC
onshiftthresh = .10; %0.10 change thresh by 10%

%offset categories and thresholds

off_thresh_above = []; off_weights = [];
on_thresh_above  = []; on_weights = [];
off_thresh_below = []; off_weights = [];
on_thresh_below  = []; on_weights = [];


%for each RFF instance, calculate offset RFF, then calculate onset RFF. Save data into performance folder.
for instancei = 1:totalinst
    aRFF.instancei = instancei;
    fric_center = pk_loc(aRFF.instancei);
    
    %%for calculating offset RFF
    aRFF.onoff = 'off';
    instancefile = [varfolname num2str(instancei) '_' aRFF.onoff '.mat'];
    
    %compute average period
    off_t0_step = round(aRFF.fs/offpitch(instancei));
    
    %first pulse located at 3Tavg + fricative location. Next pulse is
    %Tavg before.
    Tendbackpulse = fric_center + 3*off_t0_step;
    Pendbackpulse = fric_center + 3*off_t0_step;
    Tbackpulse = Tendbackpulse - off_t0_step;
    Pbackpulse = Pendbackpulse - off_t0_step;
    
    aRFF.peakcyclefull = 0; %1 if we have at least 10 peak pulses
    aRFF.troughcyclefull = 0; %1 if we have at least 10 trough pulses
    aRFF.trough_array = []; %array of trough cycle
    aRFF.peak_array = []; %array of peak cycle
    aRFF.f0y = aRFF.f0y ./ max(abs(aRFF.f0y));
    
    aRFF.PulseFindingDone = 0; %1 if quit the while loop
    aRFF.checkedonce = 0; %ran through checking_cycles at least once
    donewithdeletion = 0;
    aRFF.recordingedge = offedge(aRFF.instancei);
    
    trough_zc_all = []; %store zc
    peak_zc_all = []; %store zc
    trough_ptpratio = []; %store ptp
    peak_ptpratio = []; %store ptp

    
    while ~aRFF.PulseFindingDone
        if min(Tbackpulse, Pbackpulse)-1< aRFF.recordingedge
            plot_pulses(aRFF)
            break;
        end
        
        %finding cycle trough
        trough_l = trough_find(aRFF, Tbackpulse, Tendbackpulse);
        
        %finding cycle peak
        peak_l = peak_find(aRFF, Pbackpulse, Pendbackpulse);
        
        %if we've reached the start of the offset vowel, plot and quit
        if min(trough_l, peak_l)-off_t0_step < aRFF.recordingedge
            plot_pulses(aRFF)
            break;
        end
        
        cycle_trough_l = trough_l-off_t0_step:trough_l;
        cycle_peak_l = peak_l-off_t0_step:peak_l;
        
        %compute parameters
        troughptp = range(aRFF.f0y(cycle_trough_l)); %peak-to-peak amplitude
        trough_zc = zero_crossings(aRFF.y(cycle_trough_l)-smooth(aRFF.y(cycle_trough_l),1000)); %zero crossings
        

        peakptp = range(aRFF.f0y(cycle_peak_l)); %peak-to-peak amplitude
        peak_zc = zero_crossings(aRFF.y(cycle_peak_l)-smooth(aRFF.y(cycle_peak_l),1000)); %zero crossings
        
        
        %storing varables for new troughs
        if ~ismember(trough_l,aRFF.trough_array)
            aRFF.trough_array = [aRFF.trough_array trough_l];
            trough_zc_all = [trough_zc_all trough_zc];
            trough_ptpratio = [trough_ptpratio troughptp];
            
            last_trough_vect = aRFF.y(cycle_trough_l);
        end
        
        %storing varables for new peaks
        if ~ismember(peak_l,aRFF.peak_array)
            aRFF.peak_array = [aRFF.peak_array peak_l];
            peak_zc_all = [peak_zc_all peak_zc];
            peak_ptpratio = [peak_ptpratio peakptp];
            
            last_peak_vect = aRFF.y(cycle_peak_l);
        end
        
        %maximum period allowed is 1.5*Tavg
        max_acf_range = floor(off_t0_step*1.5);
        
        %if there are at least five pulses and the last six cycles
        %have similar shapes, the try to determine first peak
        if ~donewithdeletion & (length(trough_zc_all)>5)
            long_enough = (aRFF.trough_array(end-4) - aRFF.trough_array(end) + 1) > max_acf_range;
            if ~long_enough
                Tendbackpulse = peak_l;
                Tbackpulse = peak_l - off_t0_step;
                Pendbackpulse = trough_l;
                Pbackpulse = trough_l - off_t0_step;
                continue;
            end
            acf = autocorr(aRFF.y(aRFF.trough_array(end):aRFF.trough_array(end-4)),max_acf_range);
            maxacf = max(acf(floor(off_t0_step*0.5):max_acf_range));
            if (maxacf>0.7) && ((~diff_prob(Tbackpulse) > diff_prob(Tendbackpulse))) 
                trough_zc_thresh = (1+offshiftthresh)*threshfind(trough_zc_all);
                trough_ptp_thresh = (1-offshiftthresh)*threshfind(trough_ptpratio);
                peak_zc_thresh = (1+offshiftthresh)*threshfind(peak_zc_all);
                peak_ptp_thresh = (1-offshiftthresh)*threshfind(peak_ptpratio);
                

                deltroughzc = find(trough_zc_all>trough_zc_thresh,1,'last');
                deltroughptp = find(trough_ptpratio<trough_ptp_thresh,1,'last');
                deltrough = round(median([deltroughzc deltroughptp]));
                [deltroughmax,~] = max([deltroughzc deltroughptp]);
                
                delpeakzc = find(peak_zc_all>peak_zc_thresh,1,'last');
                delpeakptp = find(peak_ptpratio<peak_ptp_thresh,1,'last');
                delpeak = round(median([delpeakzc delpeakptp]));
                [delpeakmax,~] = max([delpeakzc delpeakptp]);
                
                trough_x = aRFF.y(aRFF.trough_array(end):aRFF.trough_array(1));
                peak_x = aRFF.y(aRFF.peak_array(end):aRFF.peak_array(1));
                trough_vals = fliplr(aRFF.trough_array);
                peak_vals = fliplr(aRFF.peak_array);
                
                [tau_trough1, tau_trough2] = evaluate4tau_split(trough_x,trough_vals);
                [tau_peak1, tau_peak2] = evaluate4tau_split(peak_x,peak_vals);
                
                cycleloc_trough1 = length(trough_vals) - tau_trough1(2);
                cycleloc_trough2 = length(trough_vals) - tau_trough2(2);
                cycleloc_peak1 = length(peak_vals) - tau_peak1(2);
                cycleloc_peak2 = length(peak_vals) - tau_peak2(2);
                

                cycleloc_trough = round(median([cycleloc_trough1 cycleloc_trough2]));
                cycleloc_peak = round(max([cycleloc_peak1 cycleloc_peak2]));


                
                for i = 1:2
                    if i == 1
                        cycleloc = cycleloc_peak;
                        PTP = peak_ptpratio;
                    else
                        cycleloc = cycleloc_trough;
                        PTP = trough_ptpratio;
                    end
                    
                    for features = 1:2
                        if ~isempty(cycleloc) && cycleloc > 0
                            cycleloc;
                        else
                            cycleloc = NaN;
                        end
                    end
                    
                    delcyc = cycleloc;
                    
                    D = delcyc(~isnan(delcyc));
                    deltroughpeakval = round(median(D));

                    if i == 1
                        if deltroughpeakval > numel(aRFF.peak_array)
                            delpeak = numel(aRFF.peak_array);
                        else
                            delpeak = deltroughpeakval;
                            if delpeak <= 0
                                delpeak = 1;
                            elseif delpeak > numel(aRFF.peak_array)
                                delpeak = numel(aRFF.peak_array);
                            end
                        end
                        if max(D) > numel(aRFF.peak_array)
                            delpeakmax = numel(aRFF.peak_array);
                        elseif max(D) < 1
                            delpeakmax = 1;
                        else
                            delpeakmax = max(D);
                        end
                    else
                        if deltroughpeakval > numel(aRFF.trough_array)
                            deltrough = numel(aRFF.trough_array);
                        else
                            deltrough = deltroughpeakval;
                            if deltrough <= 0
                                deltrough = 1;
                            elseif deltrough > numel(aRFF.trough_array)
                                deltrough = numel(aRFF.trough_array);
                            end
                        end
                        if max(D) > numel(aRFF.trough_array)
                            deltroughmax = numel(aRFF.trough_array);
                        elseif max(D) < 1
                            deltroughmax = 1;
                        else
                            deltroughmax = max(D);
                        end
                    end
                end
                
                
                %if the autocorrelation of the waveform between the mid
                %threshold and the max threshold is less than 0.3, mid
                %we want to delete up to the max threshold
                long_enough = (aRFF.trough_array(deltrough) - aRFF.trough_array(deltroughmax) + 1) > max_acf_range;
                if deltroughmax - deltrough > 3 & long_enough
                    acf = autocorr(aRFF.y(aRFF.trough_array(deltroughmax):aRFF.trough_array(deltrough)),max_acf_range);
                    maxacf = max(acf(floor(off_t0_step*0.5):max_acf_range));
                    if maxacf < 0.3
                        deltrough = deltroughmax;
                    end
                end
                
                
                %if the autocorrelation of the waveform between the mid
                %threshold and the max threshold is less than 0.3, mid
                %we want to delete up to the max threshold
                long_enough = (aRFF.peak_array(delpeak) - aRFF.peak_array(delpeakmax) + 1) > max_acf_range;
                if  delpeakmax - delpeak > 3 & long_enough
                    acf = autocorr(aRFF.y(aRFF.peak_array(delpeakmax):aRFF.peak_array(delpeak)),max_acf_range);
                    maxacf = max(acf(floor(off_t0_step*0.5):max_acf_range));
                    if maxacf < 0.3
                        delpeak = delpeakmax;
                    end
                end
                
                %if there is nothing to delete, then get more pulses
                if isnan(deltrough) | isnan(delpeak)
                    Tendbackpulse = peak_l;
                    Tbackpulse = peak_l - off_t0_step;
                    Pendbackpulse = trough_l;
                    Pbackpulse = trough_l - off_t0_step;
                    continue;
                end
                
                
                %troughtemp and peaktemp should only have pulses where
                %there are cycles
                troughtemp = aRFF.trough_array;
                peaktemp = aRFF.peak_array;
                
                troughtemp(1:min([length(troughtemp) deltrough])) = [];
                peaktemp(1:min([length(peaktemp) delpeak])) = [];

                
                %re-estimate F0
                [off_t0_step,aRFF] = finer_t0_est(Tbackpulse,aRFF,off_t0_step);
                
                %troughtemp and peaktemp is long enough, then actually
                %delete the cycles
                if (length(troughtemp) > 4) & (length(peaktemp) > 4)
                    donewithdeletion = 1;
                    aRFF.trough_array(1:min([length(aRFF.trough_array) deltrough])) = [];
                    aRFF.peak_array(1:min([length(aRFF.peak_array) delpeak])) = [];
                end
            end
        end
        %indicates that there are at least 11 trough cycle pulses
        if length(aRFF.trough_array)>10 & donewithdeletion
            aRFF.troughcyclefull = 1;
        end
        %indicates that there are at least 11 peak cycle pulses
        if length(aRFF.peak_array)>10 & donewithdeletion
            aRFF.peakcyclefull = 1;
        end
        
        if aRFF.peakcyclefull & aRFF.troughcyclefull
            aRFF = checking_cycles(aRFF,off_t0_step); %check cycles
            if isempty(aRFF.trough_array)
                break;
            end
            if length(aRFF.trough_array) < 11 & length(aRFF.peak_array) < 11
                break;
            end
            aRFF = calcRFF(aRFF); % calculate RFF
            aRFF.PulseFindingDone = 1;
        end
        
        Tendbackpulse = peak_l;
        Tbackpulse = peak_l - off_t0_step;
        
        Pendbackpulse = trough_l;
        Pbackpulse = trough_l - off_t0_step;
    end
    
    aRFF.peak_zc_all = fliplr(peak_zc_all);
    aRFF.peak_ptpratio = fliplr(peak_ptpratio);
    if ~isempty(aRFF.peak_array) & exist('peak_vals')
        con_pt = find(peak_vals == aRFF.peak_array(end));
        aRFF.peak_all_cycs = [aRFF.peak_array peak_vals(con_pt+1:end)];
    end
    
    aRFF.trough_zc_all = fliplr(trough_zc_all);
    aRFF.trough_ptpratio = fliplr(trough_ptpratio);
    if ~isempty(aRFF.trough_array) & exist('trough_vals')
        con_pt = find(trough_vals == aRFF.trough_array(end));
        aRFF.trough_all_cycs = [aRFF.trough_array trough_vals(con_pt+1:end)];
    end
    save(instancefile,'aRFF') % save RFF
    
    %clear variables.
    if isfield(aRFF,'trough_RFF_Hz')
        delfields = {'trough_RFF_Hz','peak_RFF_Hz','trough_RFF','peak_RFF','trough_RFF_meanf0','peak_RFF_meanf0'};
        aRFF = rmfield(aRFF,delfields);
    end
    
    
    
    
    
    
    
    %% for calculating onset RFF
    aRFF.onoff = 'on';
    instancefile = [varfolname num2str(instancei) '_' aRFF.onoff '.mat'];
    
    on_t0_step = round(aRFF.fs/onpitch(instancei));
    Tstartpulse = fric_center - 3*on_t0_step;
    Pstartpulse = fric_center - 3*on_t0_step;
    Tfrontpulse = Tstartpulse + on_t0_step;
    Pfrontpulse = Pstartpulse + on_t0_step;
    
    aRFF.peakcyclefull = 0;
    aRFF.troughcyclefull = 0;
    aRFF.trough_array = [];
    aRFF.peak_array = [];
    aRFF.f0y = aRFF.f0y ./ max(abs(aRFF.f0y));
    aRFF.PulseFindingDone = 0; %quit the while loop
    aRFF.checkedonce = 0;
    aRFF.recordingedge = onedge(aRFF.instancei);
    
    trough_zc_all = [];
    peak_zc_all = [];
    trough_ptpratio = [];
    peak_ptpratio = [];

    donewithdeletion = 0;
    
    while ~aRFF.PulseFindingDone
        if max(Tfrontpulse, Pfrontpulse)+1 > aRFF.recordingedge
            break;
        end
        
        %finding cycle trough
        trough_l = trough_find(aRFF, Tstartpulse, Tfrontpulse);
        
        %finding cycle peak
        peak_l = peak_find(aRFF, Pstartpulse, Pfrontpulse);
        
        if max(peak_l, trough_l)+on_t0_step > aRFF.recordingedge
            break;
        end
        
        cycle_trough_l = trough_l:trough_l+on_t0_step;
        cycle_peak_l = peak_l:peak_l+on_t0_step;
        
        troughptp = range(aRFF.f0y(cycle_trough_l));
        troughcenter = mean(aRFF.y(cycle_trough_l));
        trough_zc = zero_crossings(aRFF.y(cycle_trough_l)-smooth(aRFF.y(cycle_trough_l),1000));
        
        
        peakptp = range(aRFF.f0y(cycle_peak_l));
        peakcenter = mean(aRFF.y(cycle_peak_l));
        peak_zc = zero_crossings(aRFF.y(cycle_peak_l)-smooth(aRFF.y(cycle_peak_l),1000));

        
        if ~ismember(trough_l,aRFF.trough_array)
            aRFF.trough_array = [aRFF.trough_array trough_l];
            trough_zc_all = [trough_zc_all trough_zc];
            trough_ptpratio = [trough_ptpratio troughptp];

            last_trough_vect = aRFF.y(cycle_trough_l);
        end
        if ~ismember(peak_l,aRFF.peak_array)
            aRFF.peak_array = [aRFF.peak_array peak_l];
            peak_zc_all = [peak_zc_all peak_zc];
            peak_ptpratio = [peak_ptpratio peakptp];
            
            last_peak_vect = aRFF.y(cycle_peak_l);
        end
        
        max_acf_range = floor(on_t0_step*1.5);
        
        if ~donewithdeletion & (length(trough_ptpratio)>5) 
            long_enough = (aRFF.trough_array(end) - aRFF.trough_array(end-4) + 1) > max_acf_range;
            if ~long_enough
                Tstartpulse = peak_l;
                Tfrontpulse = peak_l + on_t0_step;
                Pstartpulse = trough_l;
                Pfrontpulse = trough_l + on_t0_step;
                continue;
            end
            acf = autocorr(aRFF.y(aRFF.trough_array(end-4):aRFF.trough_array(end)),max_acf_range);
            maxacf = max(acf(floor(on_t0_step*0.5):max_acf_range));
            if (maxacf > 0.7) & ((diff_prob(Tstartpulse) < diff_prob(Tfrontpulse)))
                
                %% Z-Score Method
                % PTP Z-Score (PEAK)
                base_mean = mean(peak_ptpratio(1:5));
                base_STD = std(peak_ptpratio(1:5));
                peak_ptpratio_zscore = (peak_ptpratio - base_mean)/base_STD;
                peak_ptpratio_zscore_norm = peak_ptpratio_zscore ./ max(abs(peak_ptpratio_zscore));
                slope_peak_ptpratio_zscore = diff(smooth(peak_ptpratio_zscore,3));
                [~,slope_ipt] = max(slope_peak_ptpratio_zscore);
                ipt = min(find(peak_ptpratio_zscore > 19))-1;
                if ~isempty(ipt)
                    slope_ipt = ipt;
                end
                
                % Adjust PTP Z-Score based on certain conditions 
                if any(peak_ptpratio_zscore > 50) & peak_ptpratio_zscore(end) > 50
                    slope_ipt = max(find(peak_ptpratio_zscore < 19));
                elseif ~any(peak_ptpratio_zscore > 50) & peak_ptpratio_zscore(end) < 50
                    [~,mloc] = max(peak_ptpratio_zscore_norm);
                    slope_ipt = max(find(peak_ptpratio_zscore_norm(1:mloc) < .16));
                end
                if any(peak_ptpratio_zscore > 220)
                    [~,mloc] = max(peak_ptpratio_zscore_norm);
                    slope_ipt = max(find(peak_ptpratio_zscore_norm(1:mloc) < 0.16));
                end
                
                if ~isempty(slope_ipt); delpeakptp = slope_ipt; else delpeakptp = 1; end
                
                % ZC Z-Score (PEAK)
                base_mean = mean(peak_zc_all(end-4:end));
                base_STD = std(peak_zc_all(end-4:end)); addon = 4;
                while base_STD == 0; addon = addon+1; base_STD = std(peak_zc_all(end-addon:end)); end
                peak_zc_zscore = (peak_zc_all - base_mean)/base_STD;
                ipt = max(find(peak_zc_zscore > 6.6));
                if ~isempty(ipt); delpeakzc = ipt; else delpeakzc = 1; end
                
                
                % PTP Z-Score (TROUGH)
                base_mean = mean(trough_ptpratio(1:5));
                base_STD = std(trough_ptpratio(1:5));
                trough_ptpratio_zscore = (trough_ptpratio - base_mean)/base_STD;
                trough_ptpratio_zscore_norm = trough_ptpratio_zscore ./ max(abs(trough_ptpratio_zscore));
                slope_trough_ptpratio_zscore = diff(smooth(trough_ptpratio_zscore,3));
                [~,slope_ipt] = max(slope_trough_ptpratio_zscore);
                ipt = min(find(trough_ptpratio_zscore > 19))-1;
                if ~isempty(ipt)
                    slope_ipt = ipt;
                end
                
                % Adjust PTP Z-Score based on certain conditions 
                if any(trough_ptpratio_zscore > 50) & trough_ptpratio_zscore(end) > 50
                    slope_ipt = max(find(trough_ptpratio_zscore < 19));
                elseif ~any(trough_ptpratio_zscore > 50) & trough_ptpratio_zscore(end) < 50
                    [~,mloc] = max(trough_ptpratio_zscore_norm);
                    slope_ipt = max(find(trough_ptpratio_zscore_norm(1:mloc) < .16));
                end
                if any(trough_ptpratio_zscore > 220)
                    [~,mloc] = max(trough_ptpratio_zscore_norm);
                    slope_ipt = max(find(trough_ptpratio_zscore_norm(1:mloc) < 0.16));
                end
                
                if ~isempty(slope_ipt); deltroughptp = slope_ipt; else deltroughptp = 1; end
                
                % ZC Z-Score (TROUGH)
                base_mean = mean(trough_zc_all(end-4:end));
                base_STD = std(trough_zc_all(end-4:end)); addon = 4;
                while base_STD == 0; addon = addon+1; base_STD = std(trough_zc_all(end-addon:end)); end
                trough_zc_zscore = (trough_zc_all - base_mean)/base_STD;
                ipt = max(find(trough_zc_zscore > 6.6));
                if ~isempty(ipt); deltroughzc = ipt; else deltroughzc = 1; end
                
                
                
                alltroughfeats = [trough_ptpratio' trough_zc_all'];
                alltroughfeats_Zscore = [trough_ptpratio_zscore' trough_zc_zscore'];
                deltrough = round(median([deltroughptp]));
                
                [deltroughmax,~] = max([deltroughptp]);
                troughfeats = {deltroughptp};
                
                allpeakfeats = [peak_ptpratio' peak_zc_all'];
                allpeakfeats_Zscore = [peak_ptpratio_zscore' peak_zc_zscore'];
                delpeak = round(median([delpeakptp]));
                
                [delpeakmax,~] = max([delpeakptp]);
                peakfeats      = {delpeakptp};
                
                
                
                %% Evaluating Tau Method
                trough_x = aRFF.y(aRFF.trough_array(1):aRFF.trough_array(end));
                peak_x = aRFF.y(aRFF.peak_array(1):aRFF.peak_array(end));
                
                trough_vals = aRFF.trough_array;
                peak_vals = aRFF.peak_array;
                
                tau_trough = evaluate4tau(trough_x,trough_vals);
                tau_peak = evaluate4tau(peak_x,peak_vals);
                
                cycleloc_trough = tau_trough(2);
                cycleloc_peak = tau_peak(2);
                %%
                
                
                for i = 1:2
                    if i == 1
                        delfeats = peakfeats;
                        cycleloc = cycleloc_peak;
                    else
                        delfeats = troughfeats;
                        cycleloc = cycleloc_trough;
                    end
                    
                    for features = 1:1
                        if ~isempty(cycleloc) && (cycleloc > 0)
                            if ~isempty(delfeats{features}) && (delfeats{features} > 0)
                                delcyc_og(features) = delfeats{features} - 1;
                            else
                                delcyc_og(features) = NaN;
                            end
                        else
                            cycleloc = NaN;
                        end
                    end
                    
                    
                    delcyc = ceil(nanmedian([cycleloc,delcyc_og]));
%                     delcyc = ceil(nanmedian([delcyc_og]));
                    D = delcyc(~isnan(delcyc));
                    deltroughpeakval = round(median(D));
                    if i == 1
                        if deltroughpeakval > numel(aRFF.peak_array)
                            delpeak = numel(aRFF.peak_array);
                        else
                            delpeak = deltroughpeakval;
                            if delpeak <= 0
                                delpeak = 1;
                            elseif delpeak > numel(aRFF.peak_array)
                                delpeak = numel(aRFF.peak_array);
                            end
                        end
                        if max(D) > numel(aRFF.peak_array)
                            delpeakmax = numel(aRFF.peak_array);
                        elseif max(D) < 1
                            delpeakmax = 1;
                        else
                            delpeakmax = max(D);
                        end
                    else
                        if deltroughpeakval > numel(aRFF.trough_array)
                            deltrough = numel(aRFF.trough_array);
                        else
                            deltrough = deltroughpeakval;
                            if deltrough <= 0
                                deltrough = 1;
                            elseif deltrough > numel(aRFF.trough_array)
                                deltrough = numel(aRFF.trough_array);
                            end
                        end
                        if max(D) > numel(aRFF.trough_array)
                            deltroughmax = numel(aRFF.trough_array);
                        elseif max(D) < 1
                            deltroughmax = 1;
                        else
                            deltroughmax = max(D);
                        end
                    end
                end
                
                if isnan(deltrough)
                    deltrough = [];
                end
                long_enough = (aRFF.trough_array(deltroughmax) - aRFF.trough_array(deltrough) + 1) > max_acf_range;
                if deltroughmax - deltrough > 3 & long_enough
                    acf = autocorr(aRFF.y(aRFF.trough_array(deltrough):aRFF.trough_array(deltroughmax)),max_acf_range);
                    maxacf = max(acf(floor(on_t0_step*0.5):floor(on_t0_step*1.5)));
                    if maxacf < 0.3
                        deltrough = deltroughmax;
                    end
                end
                
                
                if isnan(delpeak)
                    delpeak = [];
                end
                long_enough = (aRFF.peak_array(delpeakmax) - aRFF.peak_array(delpeak) + 1) > max_acf_range;
                if delpeakmax - delpeak > 3 & long_enough
                    acf = autocorr(aRFF.y(aRFF.peak_array(delpeak):aRFF.peak_array(delpeakmax)),max_acf_range);
                    maxacf = max(acf(floor(on_t0_step*0.5):floor(on_t0_step*1.5)));
                    if maxacf < 3
                        delpeak = delpeakmax;
                    end
                end
                
                if isnan(deltrough) | isnan(delpeak)
                    Tstartpulse = peak_l;
                    Tfrontpulse = peak_l + on_t0_step;
                    Pstartpulse = trough_l;
                    Pfrontpulse = trough_l + on_t0_step;
                    continue;
                end
                
%                 if min(trough_ss_all) > 0.5
%                     deltrough = length(trough_ss_all);
%                 end
%                 if min(peak_ss_all) > 0.5
%                     delpeak = length(peak_ss_all);
%                 end
                
                troughtemp = aRFF.trough_array;
                peaktemp = aRFF.peak_array;
                troughtemp(1:min([deltrough+1 length(troughtemp)])) = [];
                peaktemp(1:min([delpeak+1 length(peaktemp)])) = [];
                
                [on_t0_step,aRFF] = finer_t0_est(Tfrontpulse,aRFF,on_t0_step);
                
                if (length(troughtemp) > 4) & (length(peaktemp) > 4)
                    aRFF.trough_array(1:min([deltrough+1 length(aRFF.trough_array)])) = [];
                    aRFF.peak_array(1:min([delpeak+1 length(aRFF.peak_array)])) = [];
                    donewithdeletion = 1;
                end
            end
        end
        
        if length(aRFF.trough_array)>10 & donewithdeletion
            aRFF.troughcyclefull = 1;
        end
        
        if length(aRFF.peak_array)>10 & donewithdeletion
            aRFF.peakcyclefull = 1;
        end
        
        if aRFF.peakcyclefull & aRFF.troughcyclefull
            aRFF = checking_cycles(aRFF,on_t0_step);
            if isempty(aRFF.trough_array)
                break;
            end
            if length(aRFF.trough_array) < 11 & length(aRFF.peak_array) < 11
                break;
            end
            
            
%             curr_tvec = aRFF.y((aRFF.trough_array(1)-50):(aRFF.trough_array(1)+50));
%             curr_pvec = aRFF.y((aRFF.peak_array(1)-50):(aRFF.peak_array(1)+50));
%             [~, tshift] = min(curr_tvec);
%             [~, pshift] = max(curr_pvec);
            
%             figure
%             hold on
%             plot([-50:50],curr_tvec)
%             plot([-50:50],curr_pvec)
%             pause
%             tshift = tshift - 51; pshift = pshift - 51;
%             aRFF.trough_array(1) = aRFF.trough_array(1) + tshift;
%             aRFF.peak_array(1) = aRFF.peak_array(1) + pshift;
            
            aRFF = calcRFF(aRFF);
%             plot_pulses(aRFF);
%             
%             subplot(4,1,4); hold on;
%             if isfield(aRFF,'trough_RFF_Hz')
%                 if ~isnan(aRFF.trough_RFF_Hz)
%                     plot(14:23, aRFF.trough_RFF(1:10),'.-')
%                 else
%                     plot(14:23, aRFF.peak_RFF(1:10),'.-')
%                 end
%                 xlim([1 24])
%             end
%             h = figure(1)
%             
%             saveas(h,[folname num2str(instancei) '_' aRFF.onoff '.jpg'])
            aRFF.PulseFindingDone = 1;
        end
        
        Tstartpulse = peak_l;
        Tfrontpulse = peak_l + on_t0_step;
        Pstartpulse = trough_l;
        Pfrontpulse = trough_l + on_t0_step;
    end
    
    aRFF.peak_zc_all = peak_zc_all;
    aRFF.peak_ptpratio = peak_ptpratio;
    if ~isempty(aRFF.peak_array) & exist('peak_vals')
        con_pt = find(peak_vals == aRFF.peak_array(1));
        aRFF.peak_all_cycs = [peak_vals(1:con_pt-1) aRFF.peak_array];
    end
    aRFF.trough_zc_all = trough_zc_all;
    aRFF.trough_ptpratio = trough_ptpratio;
    if ~isempty(aRFF.trough_array) & exist('trough_vals')
        con_pt = find(trough_vals == aRFF.trough_array(1));
        aRFF.trough_all_cycs = [trough_vals(1:con_pt-1) aRFF.trough_array];
    end
    save(instancefile,'aRFF') % save RFF
   
    
    if isfield(aRFF,'trough_RFF_Hz')
        delfields = {'trough_RFF_Hz','peak_RFF_Hz','trough_RFF','peak_RFF','trough_RFF_meanf0','peak_RFF_meanf0'};
        aRFF = rmfield(aRFF,delfields);
    end
end

close all;
